from ._ApiMethod import ApiMethod
from typing import List


class Messages(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add_chat_user(self, chat_id: int, user_id: int = None, visible_messages_count: int = None):
        """
        Adds a new user to a chat.
        
        :param chat_id: Chat ID.
        :param user_id: ID of the user to be added to the chat.
        :param visible_messages_count: 
        """
    
        method_name = 'messages.addChatUser'
        return self._call(method_name, locals())

    def allow_messages_from_group(self, group_id: int, key: str = None):
        """
        Allows sending messages from community to the current user.
        
        :param group_id: Group ID.
        :param key: 
        """
    
        method_name = 'messages.allowMessagesFromGroup'
        return self._call(method_name, locals())

    def create_chat(self, user_ids: List[int] = None, title: str = None, group_id: int = None):
        """
        Creates a chat with several participants.
        
        :param user_ids: IDs of the users to be added to the chat.
        :param title: Chat title.
        :param group_id: 
        """
    
        if user_ids:
            user_ids = [str(u) for u in user_ids]
            user_ids = ','.join(user_ids)
        
        method_name = 'messages.createChat'
        return self._call(method_name, locals())

    def delete(self, message_ids: List[int] = None, spam: bool = None, group_id: int = None, delete_for_all: bool = None):
        """
        Deletes one or more messages.
        
        :param message_ids: Message IDs.
        :param spam: '1' — to mark message as spam.
        :param group_id: Group ID (for group messages with user access token)
        :param delete_for_all: '1' — delete message for for all.
        """
    
        if message_ids:
            message_ids = [str(m) for m in message_ids]
            message_ids = ','.join(message_ids)
        
        method_name = 'messages.delete'
        return self._call(method_name, locals())

    def delete_chat_photo(self, chat_id: int, group_id: int = None):
        """
        Deletes a chat's cover picture.
        
        :param chat_id: Chat ID.
        :param group_id: 
        """
    
        method_name = 'messages.deleteChatPhoto'
        return self._call(method_name, locals())

    def delete_conversation(self, user_id: int = None, peer_id: int = None, group_id: int = None):
        """
        Deletes all private messages in a conversation.
        
        :param user_id: User ID. To clear a chat history use 'chat_id'
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with user access token)
        """
    
        method_name = 'messages.deleteConversation'
        return self._call(method_name, locals())

    def deny_messages_from_group(self, group_id: int):
        """
        Denies sending message from community to the current user.
        
        :param group_id: Group ID.
        """
    
        method_name = 'messages.denyMessagesFromGroup'
        return self._call(method_name, locals())

    def edit(self, peer_id: int, message: str = None, lat: float = None, long: float = None, attachment: str = None, keep_forward_messages: bool = None, keep_snippets: bool = None, group_id: int = None, dont_parse_links: bool = None, message_id: int = None):
        """
        Edits the message.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param keep_forward_messages: '1' — to keep forwarded, messages.
        :param keep_snippets: '1' — to keep attached snippets.
        :param group_id: Group ID (for group messages with user access token)
        :param dont_parse_links: 
        :param message_id: 
        """
    
        method_name = 'messages.edit'
        return self._call(method_name, locals())

    def edit_chat(self, chat_id: int, title: str):
        """
        Edits the title of a chat.
        
        :param chat_id: Chat ID.
        :param title: New title of the chat.
        """
    
        method_name = 'messages.editChat'
        return self._call(method_name, locals())

    def get_by_conversation_message_id(self, peer_id: int, conversation_message_ids: List[int], extended: bool = None, fields: List[str] = None, group_id: int = None):
        """
        Returns messages by their IDs within the conversation.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param conversation_message_ids: Conversation message IDs.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        if conversation_message_ids:
            conversation_message_ids = [str(c) for c in conversation_message_ids]
            conversation_message_ids = ','.join(conversation_message_ids)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getByConversationMessageId'
        return self._call(method_name, locals())

    def get_by_id(self, message_ids: List[int], preview_length: int = None, extended: bool = None, fields: List[str] = None, group_id: int = None):
        """
        Returns messages by their IDs.
        
        :param message_ids: Message IDs.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        if message_ids:
            message_ids = [str(m) for m in message_ids]
            message_ids = ','.join(message_ids)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getById'
        return self._call(method_name, locals())

    def get_chat_preview(self, peer_id: int = None, link: str = None, fields: List[str] = None):
        """
        
        :param peer_id: 
        :param link: Invitation link.
        :param fields: Profile fields to return.
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getChatPreview'
        return self._call(method_name, locals())

    def get_conversation_members(self, peer_id: int, fields: List[str] = None, group_id: int = None):
        """
        Returns a list of IDs of users participating in a chat.
        
        :param peer_id: Peer ID.
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getConversationMembers'
        return self._call(method_name, locals())

    def get_conversations(self, offset: int = None, count: int = None, filter_: str = None, extended: bool = None, start_message_id: int = None, fields: List[str] = None, group_id: int = None):
        """
        Returns a list of the current user's conversations.
        
        :param offset: Offset needed to return a specific subset of conversations.
        :param count: Number of conversations to return.
        :param filter_: Filter to apply: 'all' — all conversations, 'unread' — conversations with unread messages, 'important' — conversations, marked as important (only for community messages), 'unanswered' — conversations, marked as unanswered (only for community messages)
        :param extended: '1' — return extra information about users and communities
        :param start_message_id: ID of the message from what to return dialogs.
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        if fields:
            fields = ','.join(fields)
        
        param_alias_dict = {'filter_': 'filter'}
        method_name = 'messages.getConversations'
        return self._call(method_name, locals())

    def get_conversations_by_id(self, peer_ids: List[int], extended: bool = None, fields: List[str] = None, group_id: int = None):
        """
        Returns conversations by their IDs
        
        :param peer_ids: Destination IDs. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param extended: Return extended properties
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        if peer_ids:
            peer_ids = [str(p) for p in peer_ids]
            peer_ids = ','.join(peer_ids)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getConversationsById'
        return self._call(method_name, locals())

    def get_history(self, offset: int = None, count: int = None, user_id: int = None, peer_id: int = None, start_message_id: int = None, rev: int = None, extended: bool = None, fields: List[str] = None, group_id: int = None):
        """
        Returns message history for the specified user or group chat.
        
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param user_id: ID of the user whose message history you want to return.
        :param peer_id: 
        :param start_message_id: Starting message ID from which to return history.
        :param rev: Sort order: '1' — return messages in chronological order. '0' — return messages in reverse chronological order.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getHistory'
        return self._call(method_name, locals())

    def get_history_attachments(self, peer_id: int, media_type: str = None, start_from: str = None, count: int = None, photo_sizes: bool = None, fields: List[str] = None, group_id: int = None, preserve_order: bool = None, max_forwards_level: int = None):
        """
        Returns media files from the dialog or group chat.
        
        :param peer_id: Peer ID. ", For group chat: '2000000000 + chat ID' , , For community: '-community ID'"
        :param media_type: Type of media files to return: *'photo',, *'video',, *'audio',, *'doc',, *'link'.,*'market'.,*'wall'.,*'share'
        :param start_from: Message ID to start return results from.
        :param count: Number of objects to return.
        :param photo_sizes: '1' — to return photo sizes in a
        :param fields: Additional profile [vk.com/dev/fields|fields] to return. 
        :param group_id: Group ID (for group messages with group access token)
        :param preserve_order: 
        :param max_forwards_level: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getHistoryAttachments'
        return self._call(method_name, locals())

    def get_invite_link(self, peer_id: int, reset: bool = None, group_id: int = None):
        """
        
        :param peer_id: Destination ID.
        :param reset: 1 — to generate new link (revoke previous), 0 — to return previous link.
        :param group_id: Group ID
        """
    
        method_name = 'messages.getInviteLink'
        return self._call(method_name, locals())

    def get_last_activity(self, user_id: int):
        """
        Returns a user's current status and date of last activity.
        
        :param user_id: User ID.
        """
    
        method_name = 'messages.getLastActivity'
        return self._call(method_name, locals())

    def get_long_poll_history(self, ts: int = None, pts: int = None, preview_length: int = None, onlines: bool = None, fields: List[str] = None, events_limit: int = None, msgs_limit: int = None, max_msg_id: int = None, group_id: int = None, lp_version: int = None, last_n: int = None, credentials: bool = None):
        """
        Returns updates in user's private messages.
        
        :param ts: Last value of the 'ts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param pts: Lsat value of 'pts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param onlines: '1' — to return history with online users only.
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param events_limit: Maximum number of events to return.
        :param msgs_limit: Maximum number of messages to return.
        :param max_msg_id: Maximum ID of the message among existing ones in the local copy. Both messages received with API methods (for example, , ), and data received from a Long Poll server (events with code 4) are taken into account.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version: 
        :param last_n: 
        :param credentials: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.getLongPollHistory'
        return self._call(method_name, locals())

    def get_long_poll_server(self, need_pts: bool = None, group_id: int = None, lp_version: int = None):
        """
        Returns data required for connection to a Long Poll server.
        
        :param need_pts: '1' — to return the 'pts' field, needed for the [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version: Long poll version
        """
    
        method_name = 'messages.getLongPollServer'
        return self._call(method_name, locals())

    def is_messages_from_group_allowed(self, group_id: int, user_id: int):
        """
        Returns information whether sending messages from the community to current user is allowed.
        
        :param group_id: Group ID.
        :param user_id: User ID.
        """
    
        method_name = 'messages.isMessagesFromGroupAllowed'
        return self._call(method_name, locals())

    def join_chat_by_invite_link(self, link: str):
        """
        
        :param link: Invitation link.
        """
    
        method_name = 'messages.joinChatByInviteLink'
        return self._call(method_name, locals())

    def mark_as_answered_conversation(self, peer_id: int, answered: bool = None, group_id: int = None):
        """
        Marks and unmarks conversations as unanswered.
        
        :param peer_id: ID of conversation to mark as important.
        :param answered: '1' — to mark as answered, '0' — to remove the mark
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_name = 'messages.markAsAnsweredConversation'
        return self._call(method_name, locals())

    def mark_as_important(self, message_ids: List[int] = None, important: int = None):
        """
        Marks and unmarks messages as important (starred).
        
        :param message_ids: IDs of messages to mark as important.
        :param important: '1' — to add a star (mark as important), '0' — to remove the star
        """
    
        if message_ids:
            message_ids = [str(m) for m in message_ids]
            message_ids = ','.join(message_ids)
        
        method_name = 'messages.markAsImportant'
        return self._call(method_name, locals())

    def mark_as_important_conversation(self, peer_id: int, important: bool = None, group_id: int = None):
        """
        Marks and unmarks conversations as important.
        
        :param peer_id: ID of conversation to mark as important.
        :param important: '1' — to add a star (mark as important), '0' — to remove the star
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_name = 'messages.markAsImportantConversation'
        return self._call(method_name, locals())

    def mark_as_read(self, message_ids: List[int] = None, peer_id: int = None, start_message_id: int = None, group_id: int = None):
        """
        Marks messages as read.
        
        :param message_ids: IDs of messages to mark as read.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param start_message_id: Message ID to start from.
        :param group_id: Group ID (for group messages with user access token)
        """
    
        if message_ids:
            message_ids = [str(m) for m in message_ids]
            message_ids = ','.join(message_ids)
        
        method_name = 'messages.markAsRead'
        return self._call(method_name, locals())

    def pin(self, peer_id: int, message_id: int = None):
        """
        Pin a message.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        :param message_id: 
        """
    
        method_name = 'messages.pin'
        return self._call(method_name, locals())

    def remove_chat_user(self, chat_id: int, user_id: int = None, member_id: int = None):
        """
        Allows the current user to leave a chat or, if the current user started the chat, allows the user to remove another user from the chat.
        
        :param chat_id: Chat ID.
        :param user_id: ID of the user to be removed from the chat.
        :param member_id: 
        """
    
        method_name = 'messages.removeChatUser'
        return self._call(method_name, locals())

    def restore(self, message_id: int, group_id: int = None):
        """
        Restores a deleted message.
        
        :param message_id: ID of a previously-deleted message to restore.
        :param group_id: Group ID (for group messages with user access token)
        """
    
        method_name = 'messages.restore'
        return self._call(method_name, locals())

    def search(self, q: str = None, peer_id: int = None, date: int = None, preview_length: int = None, offset: int = None, count: int = None, extended: bool = None, fields: List[str] = None, group_id: int = None):
        """
        Returns a list of the current user's private messages that match search criteria.
        
        :param q: Search query string.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param date: Date to search message before in Unixtime.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param extended: 
        :param fields: 
        :param group_id: Group ID (for group messages with group access token)
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.search'
        return self._call(method_name, locals())

    def search_conversations(self, q: str = None, count: int = None, extended: bool = None, fields: List[str] = None, group_id: int = None):
        """
        Returns a list of the current user's conversations that match search criteria.
        
        :param q: Search query string.
        :param count: Maximum number of results.
        :param extended: '1' — return extra information about users and communities
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with user access token)
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'messages.searchConversations'
        return self._call(method_name, locals())

    def send(self, user_id: int = None, random_id: int = None, peer_id: int = None, domain: str = None, chat_id: int = None, user_ids: List[int] = None, message: str = None, lat: float = None, long: float = None, attachment: str = None, reply_to: int = None, forward_messages: List[int] = None, sticker_id: int = None, group_id: int = None, keyboard: str = None, payload: str = None, dont_parse_links: bool = None, disable_mentions: bool = None, intent: str = None):
        """
        Sends a message.
        
        :param user_id: User ID (by default — current user).
        :param random_id: Unique identifier to avoid resending the message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param domain: User's short address (for example, 'illarionov').
        :param chat_id: ID of conversation the message will relate to.
        :param user_ids: IDs of message recipients (if new conversation shall be started).
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param reply_to: 
        :param forward_messages: ID of forwarded messages, separated with a comma. Listed messages of the sender will be shown in the message body at the recipient's. Example: "123,431,544"
        :param sticker_id: Sticker id.
        :param group_id: Group ID (for group messages with group access token)
        :param keyboard: 
        :param payload: 
        :param dont_parse_links: 
        :param disable_mentions: 
        :param intent: 
        """
    
        if user_ids:
            user_ids = [str(u) for u in user_ids]
            user_ids = ','.join(user_ids)
        if forward_messages:
            forward_messages = [str(f) for f in forward_messages]
            forward_messages = ','.join(forward_messages)
        
        method_name = 'messages.send'
        return self._call(method_name, locals())

    def send_message_event_answer(self, event_id: str, user_id: int, peer_id: int, event_data: dict):
        """


        :param event_id: random string, returned in message_event
        :param user_id: user id
        :param peer_id: dialog id on the groups side
        :param event_data: object action, which should occur after the button is pressed
        """

        method_name = 'messages.sendMessageEventAnswer'
        return self._call(method_name, locals())

    def set_activity(self, user_id: int = None, type_: str = None, peer_id: int = None, group_id: int = None):
        """
        Changes the status of a user as typing in a conversation.
        
        :param user_id: User ID.
        :param type_: 'typing' — user has started to type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with group access token)
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'messages.setActivity'
        return self._call(method_name, locals())

    def set_chat_photo(self, file: str):
        """
        Sets a previously-uploaded picture as the cover picture of a chat.
        
        :param file: Upload URL from the 'response' field returned by the [vk.com/dev/photos.getChatUploadServer|photos.getChatUploadServer] method upon successfully uploading an image.
        """
    
        method_name = 'messages.setChatPhoto'
        return self._call(method_name, locals())

    def unpin(self, peer_id: int, group_id: int = None):
        """
        
        :param peer_id: 
        :param group_id: 
        """
    
        method_name = 'messages.unpin'
        return self._call(method_name, locals())




import re
from ..objects.objects import BotMessage


class UpdateHandler:
    def __init__(self,
                 update_type: str,
                 filters,
                 function,
                 handled_object_type=None,
                 handler_name: str = None):

        self.update_type = update_type  # str of update type
        self.filters = filters  # list of lambda functions
        self.function = function  # function which will be executed if handler is triggered
        self.handled_object_type = handled_object_type
        self.handler_name = handler_name

    def is_triggered(self, update):
        """ checks all handler filters """
        if update['type'] != self.update_type:
            return False

        handled_object = update['object']
        for f in self.filters:
            if not f(handled_object):
                return False

        return True

    def handle(self, handled_object):
        self.function(handled_object)

    def notify(self, update, handled_object_type=None):
        if self.is_triggered(update):
            handled_object = update['object']
            if self.handled_object_type:
                handled_object = self.handled_object_type(**handled_object)
            self.handle(handled_object)


class Handlers:
    def __init__(self, debug_mode: bool, owner_id: int):
        self._debug_mode = debug_mode
        self._owner_id = owner_id
        self._handlers_list = []

    def message_new(self,
                    regexp: str = None,
                    user_id: int = None,
                    user_ids: int = None,
                    attachment_type: str = None):

        if self._debug_mode:
            user_id = self._owner_id

        def decorator(function):
            update_type = 'message_new'
            filters = []
            if regexp:
                filters.append(lambda x: True if re.fullmatch(regexp, x['message'].get('text', '')) else False)
            if user_id:
                filters.append(lambda x: x['message']['from_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['message']['from_id'] in user_ids)
            if attachment_type:
                filters.append(lambda x: attachment_type in [a['type'] for a in x['message']['attachments']])

            handler = UpdateHandler(update_type, filters, function, handled_object_type=BotMessage)
            self._handlers_list.append(handler)
            return function

        return decorator

    def message_typing_state(self,
                             user_id: int = None,
                             user_ids=None):

        if self._debug_mode:
            user_id = self._owner_id

        def decorator(function):
            update_type = 'message_typing_state'
            filters = []
            if user_id:
                filters.append(lambda x: x['from_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['from_id'] in user_ids)

            handler = UpdateHandler(update_type, filters, function)
            self._handlers_list.append(handler)
            return function

        return decorator

    def group_leave(self,
                    user_id: int = None,
                    user_ids: int = None,
                    self_: bool = None):

        if self._debug_mode:
            user_id = self._owner_id

        def decorator(function):
            update_type = 'group_leave'
            filters = []
            if user_id:
                filters.append(lambda x: x['user_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['user_id'] in user_ids)
            if self_ is not None:
                filters.append(lambda x: x['self'] == self_)

            handler = UpdateHandler(update_type, filters, function)
            self._handlers_list.append(handler)
            return function

        return decorator

    def group_join(self,
                   user_id: int = None,
                   user_ids: int = None,
                   join_type: bool = None):

        if self._debug_mode:
            user_id = self._owner_id

        def decorator(function):
            update_type = 'group_join'
            filters = []
            if user_id:
                filters.append(lambda x: x['user_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['user_id'] in user_ids)
            if join_type:
                filters.append(lambda x: x['join_type'] == join_type)

            handler = UpdateHandler(update_type, filters, function)
            self._handlers_list.append(handler)
            return function

        return decorator



from ._ApiMethod import ApiMethod
from typing import List


class Auth(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def check_phone(self, phone: str, client_id: int = None, client_secret: str = None, auth_by_phone: bool = None):
        """
        Checks a user's phone number for correctness.
        
        :param phone: Phone number.
        :param client_id: User ID.
        :param client_secret: 
        :param auth_by_phone: 
        """
    
        method_name = 'auth.checkPhone'
        return self._call(method_name, locals())

    def restore(self, phone: str, last_name: str):
        """
        Allows to restore account access using a code received via SMS. " This method is only available for apps with [vk.com/dev/auth_direct|Direct authorization] access. "
        
        :param phone: User phone number.
        :param last_name: User last name.
        """
    
        method_name = 'auth.restore'
        return self._call(method_name, locals())




from .Utils import Utils
from .Users import Users
from .Market import Market
from .Notifications import Notifications
from .Photos import Photos
from .Appwidgets import Appwidgets
from .Status import Status
from .Messages import Messages
from .Auth import Auth
from .Widgets import Widgets
from .Pages import Pages
from .Wall import Wall
from .Likes import Likes
from .Newsfeed import Newsfeed
from .Secure import Secure
from .Database import Database
from .Search import Search
from .Groups import Groups
from .Execute import Execute
from .Orders import Orders
from .Friends import Friends
from .Polls import Polls
from .Stats import Stats
from .Account import Account
from .Stories import Stories
from .Storage import Storage
from .Streaming import Streaming
from .Board import Board
from .Gifts import Gifts
from .Apps import Apps
from .Fave import Fave
from .Ads import Ads
from .Prettycards import Prettycards
from .Leads import Leads
from .Docs import Docs
from .Notes import Notes
from .Video import Video



from ._ApiMethod import ApiMethod
from typing import List


class Board(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add_topic(self, group_id: int, title: str, text: str = None, from_group: bool = None, attachments: List[str] = None):
        """
        Creates a new topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param title: Topic title.
        :param text: Text of the topic.
        :param from_group: For a community: '1' — to post the topic as by the community, '0' — to post the topic as by the user (default)
        :param attachments: List of media objects attached to the topic, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media object: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media owner. '<media_id>' — Media ID. Example: "photo100172_166443618,photo66748_265827614", , "NOTE: If you try to attach more than one reference, an error will be thrown.",
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'board.addTopic'
        return self._call(method_name, locals())

    def close_topic(self, group_id: int, topic_id: int):
        """
        Closes a topic on a community's discussion board so that comments cannot be posted.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        """
    
        method_name = 'board.closeTopic'
        return self._call(method_name, locals())

    def create_comment(self, group_id: int, topic_id: int, message: str = None, attachments: List[str] = None, from_group: bool = None, sticker_id: int = None, guid: str = None):
        """
        Adds a comment on a topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: ID of the topic to be commented on.
        :param message: (Required if 'attachments' is not set.) Text of the comment.
        :param attachments: (Required if 'text' is not set.) List of media objects attached to the comment, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media object: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media owner. '<media_id>' — Media ID.
        :param from_group: '1' — to post the comment as by the community, '0' — to post the comment as by the user (default)
        :param sticker_id: Sticker ID.
        :param guid: Unique identifier to avoid repeated comments.
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'board.createComment'
        return self._call(method_name, locals())

    def delete_comment(self, group_id: int, topic_id: int, comment_id: int):
        """
        Deletes a comment on a topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        :param comment_id: Comment ID.
        """
    
        method_name = 'board.deleteComment'
        return self._call(method_name, locals())

    def delete_topic(self, group_id: int, topic_id: int):
        """
        Deletes a topic from a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        """
    
        method_name = 'board.deleteTopic'
        return self._call(method_name, locals())

    def edit_comment(self, group_id: int, topic_id: int, comment_id: int, message: str = None, attachments: List[str] = None):
        """
        Edits a comment on a topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        :param comment_id: ID of the comment on the topic.
        :param message: (Required if 'attachments' is not set). New comment text.
        :param attachments: (Required if 'message' is not set.) List of media objects attached to the comment, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media object: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media owner. '<media_id>' — Media ID. Example: "photo100172_166443618,photo66748_265827614"
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'board.editComment'
        return self._call(method_name, locals())

    def edit_topic(self, group_id: int, topic_id: int, title: str):
        """
        Edits the title of a topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        :param title: New title of the topic.
        """
    
        method_name = 'board.editTopic'
        return self._call(method_name, locals())

    def fix_topic(self, group_id: int, topic_id: int):
        """
        Pins a topic (fixes its place) to the top of a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        """
    
        method_name = 'board.fixTopic'
        return self._call(method_name, locals())

    def get_comments(self, group_id: int, topic_id: int, need_likes: bool = None, start_comment_id: int = None, offset: int = None, count: int = None, extended: bool = None, sort: str = None):
        """
        Returns a list of comments on a topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        :param need_likes: '1' — to return the 'likes' field, '0' — not to return the 'likes' field (default)
        :param start_comment_id: 
        :param offset: Offset needed to return a specific subset of comments.
        :param count: Number of comments to return.
        :param extended: '1' — to return information about users who posted comments, '0' — to return no additional fields (default)
        :param sort: Sort order: 'asc' — by creation date in chronological order, 'desc' — by creation date in reverse chronological order,
        """
    
        method_name = 'board.getComments'
        return self._call(method_name, locals())

    def get_topics(self, group_id: int, topic_ids: List[int] = None, order: int = None, offset: int = None, count: int = None, extended: bool = None, preview: int = None, preview_length: int = None):
        """
        Returns a list of topics on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_ids: IDs of topics to be returned (100 maximum). By default, all topics are returned. If this parameter is set, the 'order', 'offset', and 'count' parameters are ignored.
        :param order: Sort order: '1' — by date updated in reverse chronological order. '2' — by date created in reverse chronological order. '-1' — by date updated in chronological order. '-2' — by date created in chronological order. If no sort order is specified, topics are returned in the order specified by the group administrator. Pinned topics are returned first, regardless of the sorting.
        :param offset: Offset needed to return a specific subset of topics.
        :param count: Number of topics to return.
        :param extended: '1' — to return information about users who created topics or who posted there last, '0' — to return no additional fields (default)
        :param preview: '1' — to return the first comment in each topic,, '2' — to return the last comment in each topic,, '0' — to return no comments. By default: '0'.
        :param preview_length: Number of characters after which to truncate the previewed comment. To preview the full comment, specify '0'.
        """
    
        if topic_ids:
            topic_ids = [str(t) for t in topic_ids]
            topic_ids = ','.join(topic_ids)
        
        method_name = 'board.getTopics'
        return self._call(method_name, locals())

    def open_topic(self, group_id: int, topic_id: int):
        """
        Re-opens a previously closed topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        """
    
        method_name = 'board.openTopic'
        return self._call(method_name, locals())

    def restore_comment(self, group_id: int, topic_id: int, comment_id: int):
        """
        Restores a comment deleted from a topic on a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        :param comment_id: Comment ID.
        """
    
        method_name = 'board.restoreComment'
        return self._call(method_name, locals())

    def unfix_topic(self, group_id: int, topic_id: int):
        """
        Unpins a pinned topic from the top of a community's discussion board.
        
        :param group_id: ID of the community that owns the discussion board.
        :param topic_id: Topic ID.
        """
    
        method_name = 'board.unfixTopic'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Stories(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def ban_owner(self, owners_ids: List[int]):
        """
        Allows to hide stories from chosen sources from current user's feed.
        
        :param owners_ids: List of sources IDs
        """
    
        if owners_ids:
            owners_ids = [str(o) for o in owners_ids]
            owners_ids = ','.join(owners_ids)
        
        method_name = 'stories.banOwner'
        return self._call(method_name, locals())

    def delete(self, owner_id: int, story_id: int):
        """
        Allows to delete story.
        
        :param owner_id: Story owner's ID. Current user id is used by default.
        :param story_id: Story ID.
        """
    
        method_name = 'stories.delete'
        return self._call(method_name, locals())

    def get(self, owner_id: int = None, extended: bool = None, fields: List[str] = None):
        """
        Returns stories available for current user.
        
        :param owner_id: Owner ID.
        :param extended: '1' — to return additional fields for users and communities. Default value is 0.
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'stories.get'
        return self._call(method_name, locals())

    def get_banned(self, extended: bool = None, fields: List[str] = None):
        """
        Returns list of sources hidden from current user's feed.
        
        :param extended: '1' — to return additional fields for users and communities. Default value is 0.
        :param fields: Additional fields to return
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'stories.getBanned'
        return self._call(method_name, locals())

    def get_by_id(self, stories: List[str], extended: bool = None, fields: List[str] = None):
        """
        Returns story by its ID.
        
        :param stories: Stories IDs separated by commas. Use format {owner_id}+'_'+{story_id}, for example, 12345_54331.
        :param extended: '1' — to return additional fields for users and communities. Default value is 0.
        :param fields: Additional fields to return
        """
    
        if stories:
            stories = ','.join(stories)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'stories.getById'
        return self._call(method_name, locals())

    def get_photo_upload_server(self, add_to_news: bool = None, user_ids: List[int] = None, reply_to_story: str = None, link_text: str = None, link_url: str = None, group_id: int = None, clickable_stickers: str = None):
        """
        Returns URL for uploading a story with photo.
        
        :param add_to_news: 1 — to add the story to friend's feed.
        :param user_ids: List of users IDs who can see the story.
        :param reply_to_story: ID of the story to reply with the current.
        :param link_text: Link text (for community's stories only).
        :param link_url: Link URL. Internal links on https://vk.com only.
        :param group_id: ID of the community to upload the story (should be verified or with the "fire" icon).
        :param clickable_stickers: 
        """
    
        if user_ids:
            user_ids = [str(u) for u in user_ids]
            user_ids = ','.join(user_ids)
        
        method_name = 'stories.getPhotoUploadServer'
        return self._call(method_name, locals())

    def get_replies(self, owner_id: int, story_id: int, access_key: str = None, extended: bool = None, fields: List[str] = None):
        """
        Returns replies to the story.
        
        :param owner_id: Story owner ID.
        :param story_id: Story ID.
        :param access_key: Access key for the private object.
        :param extended: '1' — to return additional fields for users and communities. Default value is 0.
        :param fields: Additional fields to return
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'stories.getReplies'
        return self._call(method_name, locals())

    def get_stats(self, owner_id: int, story_id: int):
        """
        Returns stories available for current user.
        
        :param owner_id: Story owner ID. 
        :param story_id: Story ID.
        """
    
        method_name = 'stories.getStats'
        return self._call(method_name, locals())

    def get_video_upload_server(self, add_to_news: bool = None, user_ids: List[int] = None, reply_to_story: str = None, link_text: str = None, link_url: str = None, group_id: int = None, clickable_stickers: str = None):
        """
        Allows to receive URL for uploading story with video.
        
        :param add_to_news: 1 — to add the story to friend's feed.
        :param user_ids: List of users IDs who can see the story.
        :param reply_to_story: ID of the story to reply with the current.
        :param link_text: Link text (for community's stories only).
        :param link_url: Link URL. Internal links on https://vk.com only.
        :param group_id: ID of the community to upload the story (should be verified or with the "fire" icon).
        :param clickable_stickers: 
        """
    
        if user_ids:
            user_ids = [str(u) for u in user_ids]
            user_ids = ','.join(user_ids)
        
        method_name = 'stories.getVideoUploadServer'
        return self._call(method_name, locals())

    def get_viewers(self, owner_id: int, story_id: int, count: int = None, offset: int = None, extended: bool = None):
        """
        Returns a list of story viewers.
        
        :param owner_id: Story owner ID.
        :param story_id: Story ID.
        :param count: Maximum number of results.
        :param offset: Offset needed to return a specific subset of results.
        :param extended: '1' — to return detailed information about photos
        """
    
        method_name = 'stories.getViewers'
        return self._call(method_name, locals())

    def hide_all_replies(self, owner_id: int, group_id: int = None):
        """
        Hides all replies in the last 24 hours from the user to current user's stories.
        
        :param owner_id: ID of the user whose replies should be hidden.
        :param group_id: 
        """
    
        method_name = 'stories.hideAllReplies'
        return self._call(method_name, locals())

    def hide_reply(self, owner_id: int, story_id: int):
        """
        Hides the reply to the current user's story.
        
        :param owner_id: ID of the user whose replies should be hidden.
        :param story_id: Story ID.
        """
    
        method_name = 'stories.hideReply'
        return self._call(method_name, locals())

    def search(self, q: str = None, place_id: int = None, latitude: float = None, longitude: float = None, radius: int = None, mentioned_id: int = None, count: int = None, extended: bool = None, fields: List[str] = None):
        """
        
        :param q: 
        :param place_id: 
        :param latitude: 
        :param longitude: 
        :param radius: 
        :param mentioned_id: 
        :param count: 
        :param extended: 
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'stories.search'
        return self._call(method_name, locals())

    def unban_owner(self, owners_ids: List[int]):
        """
        Allows to show stories from hidden sources in current user's feed.
        
        :param owners_ids: List of hidden sources to show stories from.
        """
    
        if owners_ids:
            owners_ids = [str(o) for o in owners_ids]
            owners_ids = ','.join(owners_ids)
        
        method_name = 'stories.unbanOwner'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Notes(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add(self, title: str, text: str, privacy_view: List[str] = None, privacy_comment: List[str] = None):
        """
        Creates a new note for the current user.
        
        :param title: Note title.
        :param text: Note text.
        :param privacy_view: 
        :param privacy_comment: 
        """
    
        if privacy_view:
            privacy_view = ','.join(privacy_view)
        if privacy_comment:
            privacy_comment = ','.join(privacy_comment)
        
        method_name = 'notes.add'
        return self._call(method_name, locals())

    def create_comment(self, note_id: int, message: str, owner_id: int = None, reply_to: int = None, guid: str = None):
        """
        Adds a new comment on a note.
        
        :param note_id: Note ID.
        :param message: Comment text.
        :param owner_id: Note owner ID.
        :param reply_to: ID of the user to whom the reply is addressed (if the comment is a reply to another comment).
        :param guid: 
        """
    
        method_name = 'notes.createComment'
        return self._call(method_name, locals())

    def delete(self, note_id: int):
        """
        Deletes a note of the current user.
        
        :param note_id: Note ID.
        """
    
        method_name = 'notes.delete'
        return self._call(method_name, locals())

    def delete_comment(self, comment_id: int, owner_id: int = None):
        """
        Deletes a comment on a note.
        
        :param comment_id: Comment ID.
        :param owner_id: Note owner ID.
        """
    
        method_name = 'notes.deleteComment'
        return self._call(method_name, locals())

    def edit(self, note_id: int, title: str, text: str, privacy_view: List[str] = None, privacy_comment: List[str] = None):
        """
        Edits a note of the current user.
        
        :param note_id: Note ID.
        :param title: Note title.
        :param text: Note text.
        :param privacy_view: 
        :param privacy_comment: 
        """
    
        if privacy_view:
            privacy_view = ','.join(privacy_view)
        if privacy_comment:
            privacy_comment = ','.join(privacy_comment)
        
        method_name = 'notes.edit'
        return self._call(method_name, locals())

    def edit_comment(self, comment_id: int, message: str, owner_id: int = None):
        """
        Edits a comment on a note.
        
        :param comment_id: Comment ID.
        :param message: New comment text.
        :param owner_id: Note owner ID.
        """
    
        method_name = 'notes.editComment'
        return self._call(method_name, locals())

    def get(self, note_ids: List[int] = None, user_id: int = None, offset: int = None, count: int = None, sort: int = None):
        """
        Returns a list of notes created by a user.
        
        :param note_ids: Note IDs.
        :param user_id: Note owner ID.
        :param offset: 
        :param count: Number of notes to return.
        :param sort: 
        """
    
        if note_ids:
            note_ids = [str(n) for n in note_ids]
            note_ids = ','.join(note_ids)
        
        method_name = 'notes.get'
        return self._call(method_name, locals())

    def get_by_id(self, note_id: int, owner_id: int = None, need_wiki: bool = None):
        """
        Returns a note by its ID.
        
        :param note_id: Note ID.
        :param owner_id: Note owner ID.
        :param need_wiki: 
        """
    
        method_name = 'notes.getById'
        return self._call(method_name, locals())

    def get_comments(self, note_id: int, owner_id: int = None, sort: int = None, offset: int = None, count: int = None):
        """
        Returns a list of comments on a note.
        
        :param note_id: Note ID.
        :param owner_id: Note owner ID.
        :param sort: 
        :param offset: 
        :param count: Number of comments to return.
        """
    
        method_name = 'notes.getComments'
        return self._call(method_name, locals())

    def restore_comment(self, comment_id: int, owner_id: int = None):
        """
        Restores a deleted comment on a note.
        
        :param comment_id: Comment ID.
        :param owner_id: Note owner ID.
        """
    
        method_name = 'notes.restoreComment'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Notifications(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get(self, count: int = None, start_from: str = None, filters: List[str] = None, start_time: int = None, end_time: int = None):
        """
        Returns a list of notifications about other users' feedback to the current user's wall posts.
        
        :param count: Number of notifications to return.
        :param start_from: 
        :param filters: Type of notifications to return: 'wall' — wall posts, 'mentions' — mentions in wall posts, comments, or topics, 'comments' — comments to wall posts, photos, and videos, 'likes' — likes, 'reposted' — wall posts that are copied from the current user's wall, 'followers' — new followers, 'friends' — accepted friend requests
        :param start_time: Earliest timestamp (in Unix time) of a notification to return. By default, 24 hours ago.
        :param end_time: Latest timestamp (in Unix time) of a notification to return. By default, the current time.
        """
    
        if filters:
            filters = ','.join(filters)
        
        method_name = 'notifications.get'
        return self._call(method_name, locals())

    def mark_as_viewed(self):
        """
        Resets the counter of new notifications about other users' feedback to the current user's wall posts.
        
        """
    
        method_name = 'notifications.markAsViewed'
        return self._call(method_name, locals())

    def send_message(self, user_ids: List[int], message: str, fragment: str = None, group_id: int = None):
        """
        
        :param user_ids: 
        :param message: 
        :param fragment: 
        :param group_id: 
        """
    
        if user_ids:
            user_ids = [str(u) for u in user_ids]
            user_ids = ','.join(user_ids)
        
        method_name = 'notifications.sendMessage'
        return self._call(method_name, locals())




from typing import List

import requests
import re
import logging

from .longpoll import GroupLongpoll
from .handlers import Handlers
from ..objects.objects import BotMessage
from ..methods import Messages

logger = logging.getLogger('easy_vk.bot')
logger.setLevel(logging.DEBUG)


class GroupBot:
    def __init__(self, owner_access_token: str,
                 group_access_token: str,
                 group_id: int,
                 wait: int = 25,
                 debug_mode: bool = False,
                 owner_id: int = None):
        self._owner_access_token = owner_access_token
        self._group_access_token = group_access_token
        self._group_id = group_id
        self._wait = wait
        self._debug_mode = debug_mode
        self._owner_id = owner_id

        self.handlers: Handlers = Handlers(debug_mode=self._debug_mode, owner_id=self._owner_id)

        self._longpoll = GroupLongpoll(owner_access_token, group_id, wait)

        # api methods
        session = requests.Session()
        self.messages = Messages(session, group_access_token, '5.120', 25)

    def _handle(self, update):
        for handler in self.handlers._handlers_list:
            handler.notify(update)

    def run(self):
        logger.debug(f'Run bot for group https://vk.com/public{self._group_id}\n')
        for updates in self._longpoll.listen():
            for update in updates:
                self._handle(update)



from ._ApiMethod import ApiMethod
from typing import List


class Stats(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get(self, group_id: int = None, app_id: int = None, timestamp_from: int = None, timestamp_to: int = None, interval: str = None, intervals_count: int = None, filters: List[str] = None, stats_groups: List[str] = None, extended: bool = None):
        """
        Returns statistics of a community or an application.
        
        :param group_id: Community ID.
        :param app_id: Application ID.
        :param timestamp_from: 
        :param timestamp_to: 
        :param interval: 
        :param intervals_count: 
        :param filters: 
        :param stats_groups: 
        :param extended: 
        """
    
        if filters:
            filters = ','.join(filters)
        if stats_groups:
            stats_groups = ','.join(stats_groups)
        
        method_name = 'stats.get'
        return self._call(method_name, locals())

    def get_post_reach(self, owner_id: str, post_ids: List[int]):
        """
        Returns stats for a wall post.
        
        :param owner_id: post owner community id. Specify with "-" sign.
        :param post_ids: wall posts id
        """
    
        if post_ids:
            post_ids = [str(p) for p in post_ids]
            post_ids = ','.join(post_ids)
        
        method_name = 'stats.getPostReach'
        return self._call(method_name, locals())

    def track_visitor(self, id_: str):
        """
        
        :param id_: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'stats.trackVisitor'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Appwidgets(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def update(self, code: str, type_: str):
        """
        Allows to update community app widget
        
        :param code: 
        :param type_: 
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'appWidgets.update'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Likes(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add(self, type_: str, item_id: int, owner_id: int = None, access_key: str = None):
        """
        Adds the specified object to the 'Likes' list of the current user.
        
        :param type_: Object type: 'post' — post on user or community wall, 'comment' — comment on a wall post, 'photo' — photo, 'audio' — audio, 'video' — video, 'note' — note, 'photo_comment' — comment on the photo, 'video_comment' — comment on the video, 'topic_comment' — comment in the discussion, 'sitepage' — page of the site where the [vk.com/dev/Like|Like widget] is installed
        :param item_id: Object ID.
        :param owner_id: ID of the user or community that owns the object.
        :param access_key: Access key required for an object owned by a private entity.
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'likes.add'
        return self._call(method_name, locals())

    def delete(self, type_: str, item_id: int, owner_id: int = None, access_key: str = None):
        """
        Deletes the specified object from the 'Likes' list of the current user.
        
        :param type_: Object type: 'post' — post on user or community wall, 'comment' — comment on a wall post, 'photo' — photo, 'audio' — audio, 'video' — video, 'note' — note, 'photo_comment' — comment on the photo, 'video_comment' — comment on the video, 'topic_comment' — comment in the discussion, 'sitepage' — page of the site where the [vk.com/dev/Like|Like widget] is installed
        :param item_id: Object ID.
        :param owner_id: ID of the user or community that owns the object.
        :param access_key: Access key required for an object owned by a private entity.
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'likes.delete'
        return self._call(method_name, locals())

    def get_list(self, type_: str, owner_id: int = None, item_id: int = None, page_url: str = None, filter_: str = None, friends_only: int = None, extended: bool = None, offset: int = None, count: int = None, skip_own: bool = None):
        """
        Returns a list of IDs of users who added the specified object to their 'Likes' list.
        
        :param type_: , Object type: 'post' — post on user or community wall, 'comment' — comment on a wall post, 'photo' — photo, 'audio' — audio, 'video' — video, 'note' — note, 'photo_comment' — comment on the photo, 'video_comment' — comment on the video, 'topic_comment' — comment in the discussion, 'sitepage' — page of the site where the [vk.com/dev/Like|Like widget] is installed
        :param owner_id: ID of the user, community, or application that owns the object. If the 'type' parameter is set as 'sitepage', the application ID is passed as 'owner_id'. Use negative value for a community id. If the 'type' parameter is not set, the 'owner_id' is assumed to be either the current user or the same application ID as if the 'type' parameter was set to 'sitepage'.
        :param item_id: Object ID. If 'type' is set as 'sitepage', 'item_id' can include the 'page_id' parameter value used during initialization of the [vk.com/dev/Like|Like widget].
        :param page_url: URL of the page where the [vk.com/dev/Like|Like widget] is installed. Used instead of the 'item_id' parameter.
        :param filter_: Filters to apply: 'likes' — returns information about all users who liked the object (default), 'copies' — returns information only about users who told their friends about the object
        :param friends_only: Specifies which users are returned: '1' — to return only the current user's friends, '0' — to return all users (default)
        :param extended: Specifies whether extended information will be returned. '1' — to return extended information about users and communities from the 'Likes' list, '0' — to return no additional information (default)
        :param offset: Offset needed to select a specific subset of users.
        :param count: Number of user IDs to return (maximum '1000'). Default is '100' if 'friends_only' is set to '0', otherwise, the default is '10' if 'friends_only' is set to '1'.
        :param skip_own: 
        """
    
        param_alias_dict = {'type_': 'type', 'filter_': 'filter'}
        method_name = 'likes.getList'
        return self._call(method_name, locals())

    def is_liked(self, type_: str, item_id: int, user_id: int = None, owner_id: int = None):
        """
        Checks for the object in the 'Likes' list of the specified user.
        
        :param type_: Object type: 'post' — post on user or community wall, 'comment' — comment on a wall post, 'photo' — photo, 'audio' — audio, 'video' — video, 'note' — note, 'photo_comment' — comment on the photo, 'video_comment' — comment on the video, 'topic_comment' — comment in the discussion
        :param item_id: Object ID.
        :param user_id: User ID.
        :param owner_id: ID of the user or community that owns the object.
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'likes.isLiked'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Market(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add(self, owner_id: int, name: str, description: str, category_id: int, main_photo_id: int, price: float = None, old_price: float = None, deleted: bool = None, photo_ids: List[int] = None, url: str = None):
        """
        Ads a new item to the market.
        
        :param owner_id: ID of an item owner community.
        :param name: Item name.
        :param description: Item description.
        :param category_id: Item category ID.
        :param main_photo_id: Cover photo ID.
        :param price: Item price.
        :param old_price: 
        :param deleted: Item status ('1' — deleted, '0' — not deleted).
        :param photo_ids: IDs of additional photos.
        :param url: Url for button in market item.
        """
    
        if photo_ids:
            photo_ids = [str(p) for p in photo_ids]
            photo_ids = ','.join(photo_ids)
        
        method_name = 'market.add'
        return self._call(method_name, locals())

    def add_album(self, owner_id: int, title: str, photo_id: int = None, main_album: bool = None):
        """
        Creates new collection of items
        
        :param owner_id: ID of an item owner community.
        :param title: Collection title.
        :param photo_id: Cover photo ID.
        :param main_album: Set as main ('1' – set, '0' – no).
        """
    
        method_name = 'market.addAlbum'
        return self._call(method_name, locals())

    def add_to_album(self, owner_id: int, item_id: int, album_ids: List[int]):
        """
        Adds an item to one or multiple collections.
        
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param album_ids: Collections IDs to add item to.
        """
    
        if album_ids:
            album_ids = [str(a) for a in album_ids]
            album_ids = ','.join(album_ids)
        
        method_name = 'market.addToAlbum'
        return self._call(method_name, locals())

    def create_comment(self, owner_id: int, item_id: int, message: str = None, attachments: List[str] = None, from_group: bool = None, reply_to_comment: int = None, sticker_id: int = None, guid: str = None):
        """
        Creates a new comment for an item.
        
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param message: Comment text (required if 'attachments' parameter is not specified)
        :param attachments: Comma-separated list of objects attached to a comment. The field is submitted the following way: , "'<owner_id>_<media_id>,<owner_id>_<media_id>'", , '' - media attachment type: "'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document", , '<owner_id>' - media owner id, '<media_id>' - media attachment id, , For example: "photo100172_166443618,photo66748_265827614",
        :param from_group: '1' - comment will be published on behalf of a community, '0' - on behalf of a user (by default).
        :param reply_to_comment: ID of a comment to reply with current comment to.
        :param sticker_id: Sticker ID.
        :param guid: Random value to avoid resending one comment.
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'market.createComment'
        return self._call(method_name, locals())

    def delete(self, owner_id: int, item_id: int):
        """
        Deletes an item.
        
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        """
    
        method_name = 'market.delete'
        return self._call(method_name, locals())

    def delete_album(self, owner_id: int, album_id: int):
        """
        Deletes a collection of items.
        
        :param owner_id: ID of an collection owner community.
        :param album_id: Collection ID.
        """
    
        method_name = 'market.deleteAlbum'
        return self._call(method_name, locals())

    def delete_comment(self, owner_id: int, comment_id: int):
        """
        Deletes an item's comment
        
        :param owner_id: identifier of an item owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param comment_id: comment id
        """
    
        method_name = 'market.deleteComment'
        return self._call(method_name, locals())

    def edit(self, owner_id: int, item_id: int, name: str, description: str, category_id: int, price: float, main_photo_id: int, deleted: bool = None, photo_ids: List[int] = None, url: str = None):
        """
        Edits an item.
        
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param name: Item name.
        :param description: Item description.
        :param category_id: Item category ID.
        :param price: Item price.
        :param main_photo_id: Cover photo ID.
        :param deleted: Item status ('1' — deleted, '0' — not deleted).
        :param photo_ids: IDs of additional photos.
        :param url: Url for button in market item.
        """
    
        if photo_ids:
            photo_ids = [str(p) for p in photo_ids]
            photo_ids = ','.join(photo_ids)
        
        method_name = 'market.edit'
        return self._call(method_name, locals())

    def edit_album(self, owner_id: int, album_id: int, title: str, photo_id: int = None, main_album: bool = None):
        """
        Edits a collection of items
        
        :param owner_id: ID of an collection owner community.
        :param album_id: Collection ID.
        :param title: Collection title.
        :param photo_id: Cover photo id
        :param main_album: Set as main ('1' – set, '0' – no).
        """
    
        method_name = 'market.editAlbum'
        return self._call(method_name, locals())

    def edit_comment(self, owner_id: int, comment_id: int, message: str = None, attachments: List[str] = None):
        """
        Chages item comment's text
        
        :param owner_id: ID of an item owner community.
        :param comment_id: Comment ID.
        :param message: New comment text (required if 'attachments' are not specified), , 2048 symbols maximum.
        :param attachments: Comma-separated list of objects attached to a comment. The field is submitted the following way: , "'<owner_id>_<media_id>,<owner_id>_<media_id>'", , '' - media attachment type: "'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document", , '<owner_id>' - media owner id, '<media_id>' - media attachment id, , For example: "photo100172_166443618,photo66748_265827614",
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'market.editComment'
        return self._call(method_name, locals())

    def get(self, owner_id: int, album_id: int = None, count: int = None, offset: int = None, extended: bool = None):
        """
        Returns items list for a community.
        
        :param owner_id: ID of an item owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param album_id: 
        :param count: Number of items to return.
        :param offset: Offset needed to return a specific subset of results.
        :param extended: '1' – method will return additional fields: 'likes, can_comment, car_repost, photos'. These parameters are not returned by default.
        """
    
        method_name = 'market.get'
        return self._call(method_name, locals())

    def get_album_by_id(self, owner_id: int, album_ids: List[int]):
        """
        Returns items album's data
        
        :param owner_id: identifier of an album owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param album_ids: collections identifiers to obtain data from
        """
    
        if album_ids:
            album_ids = [str(a) for a in album_ids]
            album_ids = ','.join(album_ids)
        
        method_name = 'market.getAlbumById'
        return self._call(method_name, locals())

    def get_albums(self, owner_id: int, offset: int = None, count: int = None):
        """
        Returns community's collections list.
        
        :param owner_id: ID of an items owner community.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of items to return.
        """
    
        method_name = 'market.getAlbums'
        return self._call(method_name, locals())

    def get_by_id(self, item_ids: List[str], extended: bool = None):
        """
        Returns information about market items by their ids.
        
        :param item_ids: Comma-separated ids list: {user id}_{item id}. If an item belongs to a community -{community id} is used. " 'Videos' value example: , '-4363_136089719,13245770_137352259'"
        :param extended: '1' – to return additional fields: 'likes, can_comment, car_repost, photos'. By default: '0'.
        """
    
        if item_ids:
            item_ids = ','.join(item_ids)
        
        method_name = 'market.getById'
        return self._call(method_name, locals())

    def get_categories(self, count: int = None, offset: int = None):
        """
        Returns a list of market categories.
        
        :param count: Number of results to return.
        :param offset: Offset needed to return a specific subset of results.
        """
    
        method_name = 'market.getCategories'
        return self._call(method_name, locals())

    def get_comments(self, owner_id: int, item_id: int, need_likes: bool = None, start_comment_id: int = None, offset: int = None, count: int = None, sort: str = None, extended: bool = None, fields: List[str] = None):
        """
        Returns comments list for an item.
        
        :param owner_id: ID of an item owner community
        :param item_id: Item ID.
        :param need_likes: '1' — to return likes info.
        :param start_comment_id: ID of a comment to start a list from (details below).
        :param offset: 
        :param count: Number of results to return.
        :param sort: Sort order ('asc' — from old to new, 'desc' — from new to old)
        :param extended: '1' — comments will be returned as numbered objects, in addition lists of 'profiles' and 'groups' objects will be returned.
        :param fields: List of additional profile fields to return. See the [vk.com/dev/fields|details]
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'market.getComments'
        return self._call(method_name, locals())

    def remove_from_album(self, owner_id: int, item_id: int, album_ids: List[int]):
        """
        Removes an item from one or multiple collections.
        
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param album_ids: Collections IDs to remove item from.
        """
    
        if album_ids:
            album_ids = [str(a) for a in album_ids]
            album_ids = ','.join(album_ids)
        
        method_name = 'market.removeFromAlbum'
        return self._call(method_name, locals())

    def reorder_albums(self, owner_id: int, album_id: int, before: int = None, after: int = None):
        """
        Reorders the collections list.
        
        :param owner_id: ID of an item owner community.
        :param album_id: Collection ID.
        :param before: ID of a collection to place current collection before it.
        :param after: ID of a collection to place current collection after it.
        """
    
        method_name = 'market.reorderAlbums'
        return self._call(method_name, locals())

    def reorder_items(self, owner_id: int, item_id: int, album_id: int = None, before: int = None, after: int = None):
        """
        Changes item place in a collection.
        
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param album_id: ID of a collection to reorder items in. Set 0 to reorder full items list.
        :param before: ID of an item to place current item before it.
        :param after: ID of an item to place current item after it.
        """
    
        method_name = 'market.reorderItems'
        return self._call(method_name, locals())

    def report(self, owner_id: int, item_id: int, reason: int = None):
        """
        Sends a complaint to the item.
        
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param reason: Complaint reason. Possible values: *'0' — spam,, *'1' — child porn,, *'2' — extremism,, *'3' — violence,, *'4' — drugs propaganda,, *'5' — adult materials,, *'6' — insult.
        """
    
        method_name = 'market.report'
        return self._call(method_name, locals())

    def report_comment(self, owner_id: int, comment_id: int, reason: int):
        """
        Sends a complaint to the item's comment.
        
        :param owner_id: ID of an item owner community.
        :param comment_id: Comment ID.
        :param reason: Complaint reason. Possible values: *'0' — spam,, *'1' — child porn,, *'2' — extremism,, *'3' — violence,, *'4' — drugs propaganda,, *'5' — adult materials,, *'6' — insult.
        """
    
        method_name = 'market.reportComment'
        return self._call(method_name, locals())

    def restore(self, owner_id: int, item_id: int):
        """
        Restores recently deleted item
        
        :param owner_id: ID of an item owner community.
        :param item_id: Deleted item ID.
        """
    
        method_name = 'market.restore'
        return self._call(method_name, locals())

    def restore_comment(self, owner_id: int, comment_id: int):
        """
        Restores a recently deleted comment
        
        :param owner_id: identifier of an item owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param comment_id: deleted comment id
        """
    
        method_name = 'market.restoreComment'
        return self._call(method_name, locals())

    def search(self, owner_id: int, album_id: int = None, q: str = None, price_from: int = None, price_to: int = None, tags: List[int] = None, sort: int = None, rev: int = None, offset: int = None, count: int = None, extended: bool = None, status: int = None):
        """
        Searches market items in a community's catalog
        
        :param owner_id: ID of an items owner community.
        :param album_id: 
        :param q: Search query, for example "pink slippers".
        :param price_from: Minimum item price value.
        :param price_to: Maximum item price value.
        :param tags: Comma-separated tag IDs list.
        :param sort: 
        :param rev: '0' — do not use reverse order, '1' — use reverse order
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of items to return.
        :param extended: '1' – to return additional fields: 'likes, can_comment, car_repost, photos'. By default: '0'.
        :param status: 
        """
    
        if tags:
            tags = [str(t) for t in tags]
            tags = ','.join(tags)
        
        method_name = 'market.search'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Fave(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add_article(self, url: str):
        """
        
        :param url: 
        """
    
        method_name = 'fave.addArticle'
        return self._call(method_name, locals())

    def add_link(self, link: str):
        """
        Adds a link to user faves.
        
        :param link: Link URL.
        """
    
        method_name = 'fave.addLink'
        return self._call(method_name, locals())

    def add_page(self, user_id: int = None, group_id: int = None):
        """
        
        :param user_id: 
        :param group_id: 
        """
    
        method_name = 'fave.addPage'
        return self._call(method_name, locals())

    def add_post(self, owner_id: int, id_: int, access_key: str = None):
        """
        
        :param owner_id: 
        :param id_: 
        :param access_key: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'fave.addPost'
        return self._call(method_name, locals())

    def add_product(self, owner_id: int, id_: int, access_key: str = None):
        """
        
        :param owner_id: 
        :param id_: 
        :param access_key: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'fave.addProduct'
        return self._call(method_name, locals())

    def add_tag(self, name: str = None, position: str = None):
        """
        
        :param name: 
        :param position: 
        """
    
        method_name = 'fave.addTag'
        return self._call(method_name, locals())

    def add_video(self, owner_id: int, id_: int, access_key: str = None):
        """
        
        :param owner_id: 
        :param id_: 
        :param access_key: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'fave.addVideo'
        return self._call(method_name, locals())

    def edit_tag(self, id_: int, name: str):
        """
        
        :param id_: 
        :param name: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'fave.editTag'
        return self._call(method_name, locals())

    def get(self, extended: bool = None, item_type: str = None, tag_id: int = None, offset: int = None, count: int = None, fields: str = None, is_from_snackbar: bool = None):
        """
        
        :param extended: '1' — to return additional 'wall', 'profiles', and 'groups' fields. By default: '0'.
        :param item_type: 
        :param tag_id: Tag ID.
        :param offset: Offset needed to return a specific subset of users.
        :param count: Number of users to return.
        :param fields: 
        :param is_from_snackbar: 
        """
    
        method_name = 'fave.get'
        return self._call(method_name, locals())

    def get_pages(self, offset: int = None, count: int = None, type_: str = None, fields: List[str] = None, tag_id: int = None):
        """
        
        :param offset: 
        :param count: 
        :param type_: 
        :param fields: 
        :param tag_id: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        param_alias_dict = {'type_': 'type'}
        method_name = 'fave.getPages'
        return self._call(method_name, locals())

    def get_tags(self):
        """
        
        """
    
        method_name = 'fave.getTags'
        return self._call(method_name, locals())

    def mark_seen(self):
        """
        
        """
    
        method_name = 'fave.markSeen'
        return self._call(method_name, locals())

    def remove_article(self, owner_id: int, article_id: int):
        """
        
        :param owner_id: 
        :param article_id: 
        """
    
        method_name = 'fave.removeArticle'
        return self._call(method_name, locals())

    def remove_link(self, link_id: str = None, link: str = None):
        """
        Removes link from the user's faves.
        
        :param link_id: Link ID (can be obtained by [vk.com/dev/faves.getLinks|faves.getLinks] method).
        :param link: Link URL
        """
    
        method_name = 'fave.removeLink'
        return self._call(method_name, locals())

    def remove_page(self, user_id: int = None, group_id: int = None):
        """
        
        :param user_id: 
        :param group_id: 
        """
    
        method_name = 'fave.removePage'
        return self._call(method_name, locals())

    def remove_post(self, owner_id: int, id_: int):
        """
        
        :param owner_id: 
        :param id_: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'fave.removePost'
        return self._call(method_name, locals())

    def remove_product(self, owner_id: int, id_: int):
        """
        
        :param owner_id: 
        :param id_: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'fave.removeProduct'
        return self._call(method_name, locals())

    def remove_tag(self, id_: int):
        """
        
        :param id_: 
        """
    
        param_alias_dict = {'id_': 'id'}
        method_name = 'fave.removeTag'
        return self._call(method_name, locals())

    def reorder_tags(self, ids: List[int]):
        """
        
        :param ids: 
        """
    
        if ids:
            ids = [str(i) for i in ids]
            ids = ','.join(ids)
        
        method_name = 'fave.reorderTags'
        return self._call(method_name, locals())

    def set_page_tags(self, user_id: int = None, group_id: int = None, tag_ids: List[int] = None):
        """
        
        :param user_id: 
        :param group_id: 
        :param tag_ids: 
        """
    
        if tag_ids:
            tag_ids = [str(t) for t in tag_ids]
            tag_ids = ','.join(tag_ids)
        
        method_name = 'fave.setPageTags'
        return self._call(method_name, locals())

    def set_tags(self, item_type: str = None, item_owner_id: int = None, item_id: int = None, tag_ids: List[int] = None, link_id: str = None, link_url: str = None):
        """
        
        :param item_type: 
        :param item_owner_id: 
        :param item_id: 
        :param tag_ids: 
        :param link_id: 
        :param link_url: 
        """
    
        if tag_ids:
            tag_ids = [str(t) for t in tag_ids]
            tag_ids = ','.join(tag_ids)
        
        method_name = 'fave.setTags'
        return self._call(method_name, locals())

    def track_page_interaction(self, user_id: int = None, group_id: int = None):
        """
        
        :param user_id: 
        :param group_id: 
        """
    
        method_name = 'fave.trackPageInteraction'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Wall(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def close_comments(self, owner_id: int, post_id: int):
        """
        
        :param owner_id: 
        :param post_id: 
        """
    
        method_name = 'wall.closeComments'
        return self._call(method_name, locals())

    def create_comment(self, post_id: int, owner_id: int = None, from_group: int = None, message: str = None, reply_to_comment: int = None, attachments: List[str] = None, sticker_id: int = None, guid: str = None):
        """
        Adds a comment to a post on a user wall or community wall.
        
        :param post_id: Post ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param from_group: Group ID.
        :param message: (Required if 'attachments' is not set.) Text of the comment.
        :param reply_to_comment: ID of comment to reply.
        :param attachments: (Required if 'message' is not set.) List of media objects attached to the comment, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media ojbect: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media owner. '<media_id>' — Media ID. For example: "photo100172_166443618,photo66748_265827614"
        :param sticker_id: Sticker ID.
        :param guid: Unique identifier to avoid repeated comments.
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'wall.createComment'
        return self._call(method_name, locals())

    def delete(self, owner_id: int = None, post_id: int = None):
        """
        Deletes a post from a user wall or community wall.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: ID of the post to be deleted.
        """
    
        method_name = 'wall.delete'
        return self._call(method_name, locals())

    def delete_comment(self, comment_id: int, owner_id: int = None):
        """
        Deletes a comment on a post on a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        """
    
        method_name = 'wall.deleteComment'
        return self._call(method_name, locals())

    def edit(self, post_id: int, owner_id: int = None, friends_only: bool = None, message: str = None, attachments: List[str] = None, services: str = None, signed: bool = None, publish_date: int = None, lat: float = None, long: float = None, place_id: int = None, mark_as_ads: bool = None, close_comments: bool = None, poster_bkg_id: int = None, poster_bkg_owner_id: int = None, poster_bkg_access_hash: str = None, copyright_: str = None):
        """
        Edits a post on a user wall or community wall.
        
        :param post_id: 
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param friends_only: 
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error is thrown."
        :param services: 
        :param signed: 
        :param publish_date: 
        :param lat: 
        :param long: 
        :param place_id: 
        :param mark_as_ads: 
        :param close_comments: 
        :param poster_bkg_id: 
        :param poster_bkg_owner_id: 
        :param poster_bkg_access_hash: 
        :param copyright_: 
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        param_alias_dict = {'copyright_': 'copyright'}
        method_name = 'wall.edit'
        return self._call(method_name, locals())

    def edit_ads_stealth(self, post_id: int, owner_id: int = None, message: str = None, attachments: List[str] = None, signed: bool = None, lat: float = None, long: float = None, place_id: int = None, link_button: str = None, link_title: str = None, link_image: str = None, link_video: str = None):
        """
        Allows to edit hidden post.
        
        :param post_id: Post ID. Used for publishing of scheduled and suggested posts.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'page' — wiki-page, 'note' — note, 'poll' — poll, 'album' — photo album, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' — post will be signed with the name of the posting user, '0' — post will not be signed (default)
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param place_id: ID of the location where the user was tagged.
        :param link_button: Link button ID
        :param link_title: Link title
        :param link_image: Link image url
        :param link_video: Link video ID in format "<owner_id>_<media_id>"
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'wall.editAdsStealth'
        return self._call(method_name, locals())

    def edit_comment(self, comment_id: int, owner_id: int = None, message: str = None, attachments: List[str] = None):
        """
        Edits a comment on a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param message: New comment text.
        :param attachments: List of objects attached to the comment, in the following format: , "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media attachment owner. '<media_id>' — Media attachment ID. For example: "photo100172_166443618,photo66748_265827614"
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'wall.editComment'
        return self._call(method_name, locals())

    def get(self, owner_id: int = None, domain: str = None, offset: int = None, count: int = None, filter_: str = None, extended: bool = None, fields: List[str] = None):
        """
        Returns a list of posts on a user wall or community wall.
        
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        :param domain: User or community short address.
        :param offset: Offset needed to return a specific subset of posts.
        :param count: Number of posts to return (maximum 100).
        :param filter_: Filter to apply: 'owner' — posts by the wall owner, 'others' — posts by someone else, 'all' — posts by the wall owner and others (default), 'postponed' — timed posts (only available for calls with an 'access_token'), 'suggests' — suggested posts on a community wall
        :param extended: '1' — to return 'wall', 'profiles', and 'groups' fields, '0' — to return no additional fields (default)
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        param_alias_dict = {'filter_': 'filter'}
        method_name = 'wall.get'
        return self._call(method_name, locals())

    def get_by_id(self, posts: List[str], extended: bool = None, copy_history_depth: int = None, fields: List[str] = None):
        """
        Returns a list of posts from user or community walls by their IDs.
        
        :param posts: User or community IDs and post IDs, separated by underscores. Use a negative value to designate a community ID. Example: "93388_21539,93388_20904,2943_4276,-1_1"
        :param extended: '1' — to return user and community objects needed to display posts, '0' — no additional fields are returned (default)
        :param copy_history_depth: Sets the number of parent elements to include in the array 'copy_history' that is returned if the post is a repost from another wall.
        :param fields: 
        """
    
        if posts:
            posts = ','.join(posts)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'wall.getById'
        return self._call(method_name, locals())

    def get_comment(self, comment_id: int, owner_id: int = None, extended: bool = None, fields: List[str] = None):
        """
        Returns a comment on a post on a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param extended: 
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'wall.getComment'
        return self._call(method_name, locals())

    def get_comments(self, owner_id: int = None, post_id: int = None, need_likes: bool = None, start_comment_id: int = None, offset: int = None, count: int = None, sort: str = None, preview_length: int = None, extended: bool = None, fields: List[str] = None, comment_id: int = None, thread_items_count: int = None):
        """
        Returns a list of comments on a post on a user wall or community wall.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        :param need_likes: '1' — to return the 'likes' field, '0' — not to return the 'likes' field (default)
        :param start_comment_id: 
        :param offset: Offset needed to return a specific subset of comments.
        :param count: Number of comments to return (maximum 100).
        :param sort: Sort order: 'asc' — chronological, 'desc' — reverse chronological
        :param preview_length: Number of characters at which to truncate comments when previewed. By default, '90'. Specify '0' if you do not want to truncate comments.
        :param extended: 
        :param fields: 
        :param comment_id: Comment ID.
        :param thread_items_count: Count items in threads.
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'wall.getComments'
        return self._call(method_name, locals())

    def get_reposts(self, owner_id: int = None, post_id: int = None, offset: int = None, count: int = None):
        """
        Returns information about reposts of a post on user wall or community wall.
        
        :param owner_id: User ID or community ID. By default, current user ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        :param offset: Offset needed to return a specific subset of reposts.
        :param count: Number of reposts to return.
        """
    
        method_name = 'wall.getReposts'
        return self._call(method_name, locals())

    def open_comments(self, owner_id: int, post_id: int):
        """
        
        :param owner_id: 
        :param post_id: 
        """
    
        method_name = 'wall.openComments'
        return self._call(method_name, locals())

    def pin(self, post_id: int, owner_id: int = None):
        """
        Pins the post on wall.
        
        :param post_id: Post ID.
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        """
    
        method_name = 'wall.pin'
        return self._call(method_name, locals())

    def post(self, owner_id: int = None, friends_only: bool = None, from_group: bool = None, message: str = None, attachments: List[str] = None, services: str = None, signed: bool = None, publish_date: int = None, lat: float = None, long: float = None, place_id: int = None, post_id: int = None, guid: str = None, mark_as_ads: bool = None, close_comments: bool = None, mute_notifications: bool = None, copyright_: str = None):
        """
        Adds a new post on a user wall or community wall. Can also be used to publish suggested or scheduled posts.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param friends_only: '1' — post will be available to friends only, '0' — post will be available to all users (default)
        :param from_group: For a community: '1' — post will be published by the community, '0' — post will be published by the user (default)
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'page' — wiki-page, 'note' — note, 'poll' — poll, 'album' — photo album, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param services: List of services or websites the update will be exported to, if the user has so requested. Sample values: 'twitter', 'facebook'.
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' — post will be signed with the name of the posting user, '0' — post will not be signed (default)
        :param publish_date: Publication date (in Unix time). If used, posting will be delayed until the set time.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param place_id: ID of the location where the user was tagged.
        :param post_id: Post ID. Used for publishing of scheduled and suggested posts.
        :param guid: 
        :param mark_as_ads: 
        :param close_comments: 
        :param mute_notifications: 
        :param copyright_: 
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        param_alias_dict = {'copyright_': 'copyright'}
        method_name = 'wall.post'
        return self._call(method_name, locals())

    def post_ads_stealth(self, owner_id: int, message: str = None, attachments: List[str] = None, signed: bool = None, lat: float = None, long: float = None, place_id: int = None, guid: str = None, link_button: str = None, link_title: str = None, link_image: str = None, link_video: str = None):
        """
        Allows to create hidden post which will not be shown on the community's wall and can be used for creating an ad with type "Community post".
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'page' — wiki-page, 'note' — note, 'poll' — poll, 'album' — photo album, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' — post will be signed with the name of the posting user, '0' — post will not be signed (default)
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param place_id: ID of the location where the user was tagged.
        :param guid: Unique identifier to avoid duplication the same post.
        :param link_button: Link button ID
        :param link_title: Link title
        :param link_image: Link image url
        :param link_video: Link video ID in format "<owner_id>_<media_id>"
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'wall.postAdsStealth'
        return self._call(method_name, locals())

    def report_comment(self, owner_id: int, comment_id: int, reason: int = None):
        """
        Reports (submits a complaint about) a comment on a post on a user wall or community wall.
        
        :param owner_id: ID of the user or community that owns the wall.
        :param comment_id: Comment ID.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """
    
        method_name = 'wall.reportComment'
        return self._call(method_name, locals())

    def report_post(self, owner_id: int, post_id: int, reason: int = None):
        """
        Reports (submits a complaint about) a post on a user wall or community wall.
        
        :param owner_id: ID of the user or community that owns the wall.
        :param post_id: Post ID.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """
    
        method_name = 'wall.reportPost'
        return self._call(method_name, locals())

    def repost(self, object_: str, message: str = None, group_id: int = None, mark_as_ads: bool = None, mute_notifications: bool = None):
        """
        Reposts (copies) an object to a user wall or community wall.
        
        :param object_: ID of the object to be reposted on the wall. Example: "wall66748_3675"
        :param message: Comment to be added along with the reposted object.
        :param group_id: Target community ID when reposting to a community.
        :param mark_as_ads: 
        :param mute_notifications: 
        """
    
        param_alias_dict = {'object_': 'object'}
        method_name = 'wall.repost'
        return self._call(method_name, locals())

    def restore(self, owner_id: int = None, post_id: int = None):
        """
        Restores a post deleted from a user wall or community wall.
        
        :param owner_id: User ID or community ID from whose wall the post was deleted. Use a negative value to designate a community ID.
        :param post_id: ID of the post to be restored.
        """
    
        method_name = 'wall.restore'
        return self._call(method_name, locals())

    def restore_comment(self, comment_id: int, owner_id: int = None):
        """
        Restores a comment deleted from a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        """
    
        method_name = 'wall.restoreComment'
        return self._call(method_name, locals())

    def search(self, owner_id: int = None, domain: str = None, query: str = None, owners_only: bool = None, count: int = None, offset: int = None, extended: bool = None, fields: List[str] = None):
        """
        Allows to search posts on user or community walls.
        
        :param owner_id: user or community id. "Remember that for a community 'owner_id' must be negative."
        :param domain: user or community screen name.
        :param query: search query string.
        :param owners_only: '1' – returns only page owner's posts.
        :param count: count of posts to return.
        :param offset: Offset needed to return a specific subset of posts.
        :param extended: show extended post info.
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'wall.search'
        return self._call(method_name, locals())

    def unpin(self, post_id: int, owner_id: int = None):
        """
        Unpins the post on wall.
        
        :param post_id: Post ID.
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        """
    
        method_name = 'wall.unpin'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Ads(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add_office_users(self, account_id: int, data: str):
        """
        Adds managers and/or supervisors to advertising account.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe added managers. Description of 'user_specification' objects see below.
        """
    
        method_name = 'ads.addOfficeUsers'
        return self._call(method_name, locals())

    def check_link(self, account_id: int, link_type: str, link_url: str, campaign_id: int = None):
        """
        Allows to check the ad link.
        
        :param account_id: Advertising account ID.
        :param link_type: Object type: *'community' — community,, *'post' — community post,, *'application' — VK application,, *'video' — video,, *'site' — external site.
        :param link_url: Object URL.
        :param campaign_id: Campaign ID
        """
    
        method_name = 'ads.checkLink'
        return self._call(method_name, locals())

    def create_ads(self, account_id: int, data: str):
        """
        Creates ads.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created ads. Description of 'ad_specification' objects see below.
        """
    
        method_name = 'ads.createAds'
        return self._call(method_name, locals())

    def create_campaigns(self, account_id: int, data: str):
        """
        Creates advertising campaigns.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'campaign_specification' objects see below.
        """
    
        method_name = 'ads.createCampaigns'
        return self._call(method_name, locals())

    def create_clients(self, account_id: int, data: str):
        """
        Creates clients of an advertising agency.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'client_specification' objects see below.
        """
    
        method_name = 'ads.createClients'
        return self._call(method_name, locals())

    def create_target_group(self, account_id: int, name: str, client_id: int = None, lifetime: int = None, target_pixel_id: int = None, target_pixel_rules: str = None):
        """
        Creates a group to re-target ads for users who visited advertiser's site (viewed information about the product, registered, etc.).
        
        :param account_id: Advertising account ID.
        :param name: Name of the target group — a string up to 64 characters long.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param lifetime: 'For groups with auditory created with pixel code only.', , Number of days after that users will be automatically removed from the group.
        :param target_pixel_id: 
        :param target_pixel_rules: 
        """
    
        method_name = 'ads.createTargetGroup'
        return self._call(method_name, locals())

    def delete_ads(self, account_id: int, ids: str):
        """
        Archives ads.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with ad IDs.
        """
    
        method_name = 'ads.deleteAds'
        return self._call(method_name, locals())

    def delete_campaigns(self, account_id: int, ids: str):
        """
        Archives advertising campaigns.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted campaigns.
        """
    
        method_name = 'ads.deleteCampaigns'
        return self._call(method_name, locals())

    def delete_clients(self, account_id: int, ids: str):
        """
        Archives clients of an advertising agency.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted clients.
        """
    
        method_name = 'ads.deleteClients'
        return self._call(method_name, locals())

    def delete_target_group(self, account_id: int, target_group_id: int, client_id: int = None):
        """
        Deletes a retarget group.
        
        :param account_id: Advertising account ID.
        :param target_group_id: Group ID.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """
    
        method_name = 'ads.deleteTargetGroup'
        return self._call(method_name, locals())

    def get_accounts(self):
        """
        Returns a list of advertising accounts.
        
        """
    
        method_name = 'ads.getAccounts'
        return self._call(method_name, locals())

    def get_ads(self, account_id: int, ad_ids: str = None, campaign_ids: str = None, client_id: int = None, include_deleted: bool = None, limit: int = None, offset: int = None):
        """
        Returns number of ads.
        
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'Available and required for advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown: *0 — show only active ads,, *1 — show all ads.
        :param limit: Limit of number of returned ads. Used only if ad_ids parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        """
    
        method_name = 'ads.getAds'
        return self._call(method_name, locals())

    def get_ads_layout(self, account_id: int, ad_ids: str = None, campaign_ids: str = None, client_id: int = None, include_deleted: bool = None, limit: int = None, offset: int = None):
        """
        Returns descriptions of ad layouts.
        
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 — show only active ads,, *1 — show all ads.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        """
    
        method_name = 'ads.getAdsLayout'
        return self._call(method_name, locals())

    def get_ads_targeting(self, account_id: int, ad_ids: str = None, campaign_ids: str = None, client_id: int = None, include_deleted: bool = None, limit: int = None, offset: int = None):
        """
        Returns ad targeting parameters.
        
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: flag that specifies whether archived ads shall be shown: *0 — show only active ads,, *1 — show all ads.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset needed to return a specific subset of results.
        """
    
        method_name = 'ads.getAdsTargeting'
        return self._call(method_name, locals())

    def get_budget(self, account_id: int):
        """
        Returns current budget of the advertising account.
        
        :param account_id: Advertising account ID.
        """
    
        method_name = 'ads.getBudget'
        return self._call(method_name, locals())

    def get_campaigns(self, account_id: int, client_id: int = None, include_deleted: bool = None, campaign_ids: str = None, fields: List[str] = None):
        """
        Returns a list of campaigns in an advertising account.
        
        :param account_id: Advertising account ID.
        :param client_id: 'For advertising agencies'. ID of the client advertising campaigns are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 — show only active campaigns,, *1 — show all campaigns.
        :param campaign_ids: Filter of advertising campaigns to show. Serialized JSON array with campaign IDs. Only campaigns that exist in 'campaign_ids' and belong to the specified advertising account will be shown. If the parameter is null, all campaigns will be shown.
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'ads.getCampaigns'
        return self._call(method_name, locals())

    def get_categories(self, lang: str = None):
        """
        Returns a list of possible ad categories.
        
        :param lang: Language. The full list of supported languages is [vk.com/dev/api_requests|here].
        """
    
        method_name = 'ads.getCategories'
        return self._call(method_name, locals())

    def get_clients(self, account_id: int):
        """
        Returns a list of advertising agency's clients.
        
        :param account_id: Advertising account ID.
        """
    
        method_name = 'ads.getClients'
        return self._call(method_name, locals())

    def get_demographics(self, account_id: int, ids_type: str, ids: str, period: str, date_from: str, date_to: str):
        """
        Returns demographics for ads or campaigns.
        
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param period: Data grouping by dates: *day — statistics by days,, *month — statistics by months,, *overall — overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — day it was created on,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — current day,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — current month,, *overall: 0.
        """
    
        method_name = 'ads.getDemographics'
        return self._call(method_name, locals())

    def get_flood_stats(self, account_id: int):
        """
        Returns information about current state of a counter — number of remaining runs of methods and time to the next counter nulling in seconds.
        
        :param account_id: Advertising account ID.
        """
    
        method_name = 'ads.getFloodStats'
        return self._call(method_name, locals())

    def get_office_users(self, account_id: int):
        """
        Returns a list of managers and supervisors of advertising account.
        
        :param account_id: Advertising account ID.
        """
    
        method_name = 'ads.getOfficeUsers'
        return self._call(method_name, locals())

    def get_posts_reach(self, account_id: int, ids_type: str, ids: str):
        """
        Returns detailed statistics of promoted posts reach from campaigns and ads.
        
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 100 objects.
        """
    
        method_name = 'ads.getPostsReach'
        return self._call(method_name, locals())

    def get_rejection_reason(self, account_id: int, ad_id: int):
        """
        Returns a reason of ad rejection for pre-moderation.
        
        :param account_id: Advertising account ID.
        :param ad_id: Ad ID.
        """
    
        method_name = 'ads.getRejectionReason'
        return self._call(method_name, locals())

    def get_statistics(self, account_id: int, ids_type: str, ids: str, period: str, date_from: str, date_to: str, stats_fields: List[str] = None):
        """
        Returns statistics of performance indicators for ads, campaigns, clients or the whole account.
        
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns,, *client — clients,, *office — account.
        :param ids: IDs requested ads, campaigns, clients or account, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param period: Data grouping by dates: *day — statistics by days,, *month — statistics by months,, *overall — overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — day it was created on,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — current day,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — current month,, *overall: 0.
        :param stats_fields: Additional fields to add to statistics
        """
    
        if stats_fields:
            stats_fields = ','.join(stats_fields)
        
        method_name = 'ads.getStatistics'
        return self._call(method_name, locals())

    def get_suggestions(self, section: str, ids: str = None, q: str = None, country: int = None, cities: str = None, lang: str = None):
        """
        Returns a set of auto-suggestions for various targeting parameters.
        
        :param section: Section, suggestions are retrieved in. Available values: *countries — request of a list of countries. If q is not set or blank, a short list of countries is shown. Otherwise, a full list of countries is shown. *regions — requested list of regions. 'country' parameter is required. *cities — requested list of cities. 'country' parameter is required. *districts — requested list of districts. 'cities' parameter is required. *stations — requested list of subway stations. 'cities' parameter is required. *streets — requested list of streets. 'cities' parameter is required. *schools — requested list of educational organizations. 'cities' parameter is required. *interests — requested list of interests. *positions — requested list of positions (professions). *group_types — requested list of group types. *religions — requested list of religious commitments. *browsers — requested list of browsers and mobile devices.
        :param ids: Objects IDs separated by commas. If the parameter is passed, 'q, country, cities' should not be passed.
        :param q: Filter-line of the request (for countries, regions, cities, streets, schools, interests, positions).
        :param country: ID of the country objects are searched in.
        :param cities: IDs of cities where objects are searched in, separated with a comma.
        :param lang: Language of the returned string values. Supported languages: *ru — Russian,, *ua — Ukrainian,, *en — English.
        """
    
        method_name = 'ads.getSuggestions'
        return self._call(method_name, locals())

    def get_target_groups(self, account_id: int, client_id: int = None, extended: bool = None):
        """
        Returns a list of target groups.
        
        :param account_id: Advertising account ID.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param extended: '1' — to return pixel code.
        """
    
        method_name = 'ads.getTargetGroups'
        return self._call(method_name, locals())

    def get_targeting_stats(self, account_id: int, link_url: str, client_id: int = None, criteria: str = None, ad_id: int = None, ad_format: int = None, ad_platform: str = None, ad_platform_no_wall: str = None, ad_platform_no_ad_network: str = None, link_domain: str = None):
        """
        Returns the size of targeting audience, and also recommended values for CPC and CPM.
        
        :param account_id: Advertising account ID.
        :param link_url: URL for the advertised object.
        :param client_id: 
        :param criteria: Serialized JSON object that describes targeting parameters. Description of 'criteria' object see below.
        :param ad_id: ID of an ad which targeting parameters shall be analyzed.
        :param ad_format: Ad format. Possible values: *'1' — image and text,, *'2' — big image,, *'3' — exclusive format,, *'4' — community, square image,, *'7' — special app format,, *'8' — special community format,, *'9' — post in community,, *'10' — app board.
        :param ad_platform: Platforms to use for ad showing. Possible values: (for 'ad_format' = '1'), *'0' — VK and partner sites,, *'1' — VK only. (for 'ad_format' = '9'), *'all' — all platforms,, *'desktop' — desktop version,, *'mobile' — mobile version and apps.
        :param ad_platform_no_wall: 
        :param ad_platform_no_ad_network: 
        :param link_domain: Domain of the advertised object.
        """
    
        method_name = 'ads.getTargetingStats'
        return self._call(method_name, locals())

    def get_upload_ur_l(self, ad_format: int, icon: int = None):
        """
        Returns URL to upload an ad photo to.
        
        :param ad_format: Ad format: *1 — image and text,, *2 — big image,, *3 — exclusive format,, *4 — community, square image,, *7 — special app format.
        :param icon: 
        """
    
        method_name = 'ads.getUploadURL'
        return self._call(method_name, locals())

    def get_video_upload_ur_l(self):
        """
        Returns URL to upload an ad video to.
        
        """
    
        method_name = 'ads.getVideoUploadURL'
        return self._call(method_name, locals())

    def import_target_contacts(self, account_id: int, target_group_id: int, contacts: str, client_id: int = None):
        """
        Imports a list of advertiser's contacts to count VK registered users against the target group.
        
        :param account_id: Advertising account ID.
        :param target_group_id: Target group ID.
        :param contacts: List of phone numbers, emails or user IDs separated with a comma.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """
    
        method_name = 'ads.importTargetContacts'
        return self._call(method_name, locals())

    def remove_office_users(self, account_id: int, ids: str):
        """
        Removes managers and/or supervisors from advertising account.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted managers.
        """
    
        method_name = 'ads.removeOfficeUsers'
        return self._call(method_name, locals())

    def update_ads(self, account_id: int, data: str):
        """
        Edits ads.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in ads. Description of 'ad_edit_specification' objects see below.
        """
    
        method_name = 'ads.updateAds'
        return self._call(method_name, locals())

    def update_campaigns(self, account_id: int, data: str):
        """
        Edits advertising campaigns.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in campaigns. Description of 'campaign_mod' objects see below.
        """
    
        method_name = 'ads.updateCampaigns'
        return self._call(method_name, locals())

    def update_clients(self, account_id: int, data: str):
        """
        Edits clients of an advertising agency.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in clients. Description of 'client_mod' objects see below.
        """
    
        method_name = 'ads.updateClients'
        return self._call(method_name, locals())

    def update_target_group(self, account_id: int, target_group_id: int, name: str, client_id: int = None, domain: str = None, lifetime: int = None, target_pixel_id: int = None, target_pixel_rules: str = None):
        """
        Edits a retarget group.
        
        :param account_id: Advertising account ID.
        :param target_group_id: Group ID.
        :param name: New name of the target group — a string up to 64 characters long.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        :param domain: Domain of the site where user accounting code will be placed.
        :param lifetime: 'Only for the groups that get audience from sites with user accounting code.', Time in days when users added to a retarget group will be automatically excluded from it. '0' – automatic exclusion is off.
        :param target_pixel_id: 
        :param target_pixel_rules: 
        """
    
        method_name = 'ads.updateTargetGroup'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Orders(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def cancel_subscription(self, user_id: int, subscription_id: int, pending_cancel: bool = None):
        """
        
        :param user_id: 
        :param subscription_id: 
        :param pending_cancel: 
        """
    
        method_name = 'orders.cancelSubscription'
        return self._call(method_name, locals())

    def change_state(self, order_id: int, action: str, app_order_id: int = None, test_mode: bool = None):
        """
        Changes order status.
        
        :param order_id: order ID.
        :param action: action to be done with the order. Available actions: *cancel — to cancel unconfirmed order. *charge — to confirm unconfirmed order. Applies only if processing of [vk.com/dev/payments_status|order_change_state] notification failed. *refund — to cancel confirmed order.
        :param app_order_id: internal ID of the order in the application.
        :param test_mode: if this parameter is set to 1, this method returns a list of test mode orders. By default — 0.
        """
    
        method_name = 'orders.changeState'
        return self._call(method_name, locals())

    def get(self, offset: int = None, count: int = None, test_mode: bool = None):
        """
        Returns a list of orders.
        
        :param offset: 
        :param count: number of returned orders.
        :param test_mode: if this parameter is set to 1, this method returns a list of test mode orders. By default — 0.
        """
    
        method_name = 'orders.get'
        return self._call(method_name, locals())

    def get_amount(self, user_id: int, votes: List[str]):
        """
        
        :param user_id: 
        :param votes: 
        """
    
        if votes:
            votes = ','.join(votes)
        
        method_name = 'orders.getAmount'
        return self._call(method_name, locals())

    def get_by_id(self, order_id: int = None, order_ids: List[int] = None, test_mode: bool = None):
        """
        Returns information about orders by their IDs.
        
        :param order_id: order ID.
        :param order_ids: order IDs (when information about several orders is requested).
        :param test_mode: if this parameter is set to 1, this method returns a list of test mode orders. By default — 0.
        """
    
        if order_ids:
            order_ids = [str(o) for o in order_ids]
            order_ids = ','.join(order_ids)
        
        method_name = 'orders.getById'
        return self._call(method_name, locals())

    def get_user_subscription_by_id(self, user_id: int, subscription_id: int):
        """
        
        :param user_id: 
        :param subscription_id: 
        """
    
        method_name = 'orders.getUserSubscriptionById'
        return self._call(method_name, locals())

    def get_user_subscriptions(self, user_id: int):
        """
        
        :param user_id: 
        """
    
        method_name = 'orders.getUserSubscriptions'
        return self._call(method_name, locals())

    def update_subscription(self, user_id: int, subscription_id: int, price: int):
        """
        
        :param user_id: 
        :param subscription_id: 
        :param price: 
        """
    
        method_name = 'orders.updateSubscription'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Utils(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def check_link(self, url: str):
        """
        Checks whether a link is blocked in VK.
        
        :param url: Link to check (e.g., 'http://google.com').
        """
    
        method_name = 'utils.checkLink'
        return self._call(method_name, locals())

    def delete_from_last_shortened(self, key: str):
        """
        Deletes shortened link from user's list.
        
        :param key: Link key (characters after vk.cc/).
        """
    
        method_name = 'utils.deleteFromLastShortened'
        return self._call(method_name, locals())

    def get_last_shortened_links(self, count: int = None, offset: int = None):
        """
        Returns a list of user's shortened links.
        
        :param count: Number of links to return.
        :param offset: Offset needed to return a specific subset of links.
        """
    
        method_name = 'utils.getLastShortenedLinks'
        return self._call(method_name, locals())

    def get_link_stats(self, key: str, source: str = None, access_key: str = None, interval: str = None, intervals_count: int = None, extended: bool = None):
        """
        Returns stats data for shortened link.
        
        :param key: Link key (characters after vk.cc/).
        :param source: Source of scope
        :param access_key: Access key for private link stats.
        :param interval: Interval.
        :param intervals_count: Number of intervals to return.
        :param extended: 1 — to return extended stats data (sex, age, geo). 0 — to return views number only.
        """
    
        method_name = 'utils.getLinkStats'
        return self._call(method_name, locals())

    def get_server_time(self):
        """
        Returns the current time of the VK server.
        
        """
    
        method_name = 'utils.getServerTime'
        return self._call(method_name, locals())

    def get_short_link(self, url: str, private: bool = None):
        """
        Allows to receive a link shortened via vk.cc.
        
        :param url: URL to be shortened.
        :param private: 1 — private stats, 0 — public stats.
        """
    
        method_name = 'utils.getShortLink'
        return self._call(method_name, locals())

    def resolve_screen_name(self, screen_name: str):
        """
        Detects a type of object (e.g., user, community, application) and its ID by screen name.
        
        :param screen_name: Screen name of the user, community (e.g., 'apiclub,' 'andrew', or 'rules_of_war'), or application.
        """
    
        method_name = 'utils.resolveScreenName'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Execute(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def execute(self, code: str = None):
        """
        
        :param code: 
        """
    
        method_name = 'execute'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Prettycards(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def create(self, owner_id: int, photo: str, title: str, link: str, price: str = None, price_old: str = None, button: str = None):
        """
        
        :param owner_id: 
        :param photo: 
        :param title: 
        :param link: 
        :param price: 
        :param price_old: 
        :param button: 
        """
    
        method_name = 'prettyCards.create'
        return self._call(method_name, locals())

    def delete(self, owner_id: int, card_id: int):
        """
        
        :param owner_id: 
        :param card_id: 
        """
    
        method_name = 'prettyCards.delete'
        return self._call(method_name, locals())

    def edit(self, owner_id: int, card_id: int, photo: str = None, title: str = None, link: str = None, price: str = None, price_old: str = None, button: str = None):
        """
        
        :param owner_id: 
        :param card_id: 
        :param photo: 
        :param title: 
        :param link: 
        :param price: 
        :param price_old: 
        :param button: 
        """
    
        method_name = 'prettyCards.edit'
        return self._call(method_name, locals())

    def get(self, owner_id: int, offset: int = None, count: int = None):
        """
        
        :param owner_id: 
        :param offset: 
        :param count: 
        """
    
        method_name = 'prettyCards.get'
        return self._call(method_name, locals())

    def get_by_id(self, owner_id: int, card_ids: List[int]):
        """
        
        :param owner_id: 
        :param card_ids: 
        """
    
        if card_ids:
            card_ids = [str(c) for c in card_ids]
            card_ids = ','.join(card_ids)
        
        method_name = 'prettyCards.getById'
        return self._call(method_name, locals())

    def get_upload_ur_l(self):
        """
        
        """
    
        method_name = 'prettyCards.getUploadURL'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Gifts(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get(self, user_id: int = None, count: int = None, offset: int = None):
        """
        Returns a list of user gifts.
        
        :param user_id: User ID.
        :param count: Number of gifts to return.
        :param offset: Offset needed to return a specific subset of results.
        """
    
        method_name = 'gifts.get'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Polls(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add_vote(self, poll_id: int, answer_ids: List[int], owner_id: int = None, is_board: bool = None):
        """
        Adds the current user's vote to the selected answer in the poll.
        
        :param poll_id: Poll ID.
        :param answer_ids: 
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board: 
        """
    
        if answer_ids:
            answer_ids = [str(a) for a in answer_ids]
            answer_ids = ','.join(answer_ids)
        
        method_name = 'polls.addVote'
        return self._call(method_name, locals())

    def create(self, question: str = None, is_anonymous: bool = None, is_multiple: bool = None, end_date: int = None, owner_id: int = None, add_answers: str = None, photo_id: int = None, background_id: str = None):
        """
        Creates polls that can be attached to the users' or communities' posts.
        
        :param question: question text
        :param is_anonymous: '1' – anonymous poll, participants list is hidden,, '0' – public poll, participants list is available,, Default value is '0'.
        :param is_multiple: 
        :param end_date: 
        :param owner_id: If a poll will be added to a communty it is required to send a negative group identifier. Current user by default.
        :param add_answers: available answers list, for example: " ["yes","no","maybe"]", There can be from 1 to 10 answers.
        :param photo_id: 
        :param background_id: 
        """
    
        method_name = 'polls.create'
        return self._call(method_name, locals())

    def delete_vote(self, poll_id: int, answer_id: int, owner_id: int = None, is_board: bool = None):
        """
        Deletes the current user's vote from the selected answer in the poll.
        
        :param poll_id: Poll ID.
        :param answer_id: Answer ID.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board: 
        """
    
        method_name = 'polls.deleteVote'
        return self._call(method_name, locals())

    def edit(self, poll_id: int, owner_id: int = None, question: str = None, add_answers: str = None, edit_answers: str = None, delete_answers: str = None, end_date: int = None, photo_id: int = None, background_id: str = None):
        """
        Edits created polls
        
        :param poll_id: edited poll's id
        :param owner_id: poll owner id
        :param question: new question text
        :param add_answers: answers list, for example: , "["yes","no","maybe"]"
        :param edit_answers: object containing answers that need to be edited,, key – answer id, value – new answer text. Example: {"382967099":"option1", "382967103":"option2"}"
        :param delete_answers: list of answer ids to be deleted. For example: "[382967099, 382967103]"
        :param end_date: 
        :param photo_id: 
        :param background_id: 
        """
    
        method_name = 'polls.edit'
        return self._call(method_name, locals())

    def get_by_id(self, poll_id: int, owner_id: int = None, is_board: bool = None, extended: bool = None, friends_count: int = None, fields: List[str] = None, name_case: str = None):
        """
        Returns detailed information about a poll by its ID.
        
        :param poll_id: Poll ID.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board: '1' – poll is in a board, '0' – poll is on a wall. '0' by default.
        :param extended: 
        :param friends_count: 
        :param fields: 
        :param name_case: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'polls.getById'
        return self._call(method_name, locals())

    def get_voters(self, poll_id: int, answer_ids: List[int], owner_id: int = None, is_board: bool = None, friends_only: bool = None, offset: int = None, count: int = None, fields: List[str] = None, name_case: str = None):
        """
        Returns a list of IDs of users who selected specific answers in the poll.
        
        :param poll_id: Poll ID.
        :param answer_ids: Answer IDs.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board: 
        :param friends_only: '1' — to return only current user's friends, '0' — to return all users (default),
        :param offset: Offset needed to return a specific subset of voters. '0' — (default)
        :param count: Number of user IDs to return (if the 'friends_only' parameter is not set, maximum '1000', otherwise '10'). '100' — (default)
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate (birthdate)', 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'rate', 'contacts', 'education', 'online', 'counters'.
        :param name_case: Case for declension of user name and surname: , 'nom' — nominative (default) , 'gen' — genitive , 'dat' — dative , 'acc' — accusative , 'ins' — instrumental , 'abl' — prepositional
        """
    
        if answer_ids:
            answer_ids = [str(a) for a in answer_ids]
            answer_ids = ','.join(answer_ids)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'polls.getVoters'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Search(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get_hints(self, q: str = None, offset: int = None, limit: int = None, filters: List[str] = None, fields: List[str] = None, search_global: bool = None):
        """
        Allows the programmer to do a quick search for any substring.
        
        :param q: Search query string.
        :param offset: Offset for querying specific result subset
        :param limit: Maximum number of results to return.
        :param filters: 
        :param fields: 
        :param search_global: 
        """
    
        if filters:
            filters = ','.join(filters)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'search.getHints'
        return self._call(method_name, locals())

